<?php
/**
*   Plugin Name: TB Testimonials
*   Plugin URI: http://travisballard.com/wordpress/tb-testimonials/
*   Description: Testimonials managed by Custom Post Types. Supports a testimonial.php template file for single testimonial pages. Testimonial Shortcode to insert testimonials in any post. Ajax Scrolling Sidebar Widget
*   Version: 1.4.4
*   Author: Travis Ballard
*   Author URI: http://www.travisballard.com
*
*   Copyright 2010 Travis Ballard
*/
/*******************************************************************************
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.

*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.

*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

class TBTestimonials
{
    var $settings,
        $load_codemirror_js = false;

    /**
    * constructor
    *
    */
    function __construct()
    {
        global $pagenow;

        # settings
        add_action( 'admin_init', array( &$this, 'register_settings' ) );
        $this->settings = tbtestimonials_settings();

        # check for quote bug where we had <div class="testimonial-data""> with the extra " in the deffault output.
        # exists in versions installed prior to version 1.4
        if( $this->quote_bug_exists() ) $this->fix_quote_bug();

        # update messages
        if( is_admin() ) add_filter( 'post_updated_messages', array( &$this, 'post_update_messages' ) );

        # add in a testimonial template for people that want to link to them on their own pages
        if( ! is_admin() ) add_action( 'template_redirect', array( &$this, 'maybe_load_testimonial_template' ) );

        # meta boxes
        if( is_admin() ) add_action( 'admin_menu', array( &$this, 'add_meta_boxes' ) );

        # save meta box data
        add_action( 'save_post', array( &$this, 'save_postdata' ) );

        # settings menu
        add_action( 'admin_menu', array( &$this, 'add_settings_menu' ) );

        # ajax stuff
        add_action( 'wp_ajax_tbtestimonials-add-category', array( &$this, 'ajax_add_category' ) );
        add_action( 'wp_ajax_tbtestimonials-delete-category', array( &$this, 'ajax_delete_category' ) );

        # shortcode
        add_action('wp_ajax_tbtestimonials-shortcode', array( &$this, 'shortcode_window' ) );

        add_shortcode( 'testimonial', array( &$this, 'shortcode' ) );
        add_shortcode( 'testimonial_form', array( &$this, 'testimonial_form' ) );

        # admin js
        $pages = array( 'edit.php', 'post-new.php', 'post.php' );
        if( in_array( $pagenow, $pages ) )
        {
            wp_enqueue_script( 'tbtestimonials_admin', plugins_url( 'inc/js/tbtestimonials.admin.js', __FILE__ ), array( 'jquery' ), '1.0' );
            wp_localize_script( 'tbtestimonials_admin', 'tbtestimonials_admin', array( 'add_category_ajaxurl' => admin_url( 'admin-ajax.php?action=tbtestimonials-add-category' ), 'delete_category_ajaxurl' => admin_url( 'admin-ajax.php?action=tbtestimonials-delete-category' ) ) );
        }

        # admin css
        if( is_admin() ) wp_enqueue_style( 'tbtestimonials-admin-stylesheet', plugins_url( 'inc/css/tbtestimonials.admin.css', __FILE__ ), array(), '1.0', 'screen' );

        # code mirror
        wp_enqueue_script( 'CodeMirror', plugins_url( 'inc/js/codemirror.js', __FILE__ ), array(), '1.0' );

        if( $pagenow == 'edit.php' && isset( $_GET['post_type'] ) && $_GET['post_type'] == 'testimonial' && isset( $_GET['page'] ) && $_GET['page'] == 'tbtestimonials-syntax-settings' )
            add_action ('admin_footer', array( &$this, 'add_codemirror' ) );

        # only call when widget is active
        if( is_active_widget( false, false, 'tbtestimonialswidget' ) )
        {
            # load js
            if( ! is_admin() ) wp_enqueue_script( 'tbtestimonials', plugins_url( 'inc/js/tbtestimonials.js', __FILE__ ), array( 'jquery' ), '1.0' );

            # load css
            if( ! is_admin() && isset( $this->settings['use_stylesheet'] ) ) wp_enqueue_style( 'tbtestimonials-stylesheet', plugins_url( 'inc/css/tbtestimonials.css', __FILE__ ), array(), '1.0', 'screen' );

            # ajax for widget
            add_action( 'wp_ajax_nopriv_tbtestimonials', array( &$this, 'ajax_get_testimonials' ) );
            add_action( 'wp_ajax_tbtestimonials', array( &$this, 'ajax_get_testimonials' ) );
            wp_localize_script( 'tbtestimonials', 'tbtestimonials', array( 'ajaxurl' => admin_url( 'admin-ajax.php?action=tbtestimonials' ) ) );
        }

        # add new post type
        register_post_type(
            'testimonial',
            array(
                'labels' => array(
                    'name' => 'Testimonials',
                    'singular_name' => 'Testimonial',
                    'add_new' => 'Add A Testimonial',
                    'add_new_item' => 'Add A Testimonial',
                    'edit_item' => 'Edit Testimonial',
                    'new_item' => 'New Testimonial',
                    'view_item' => 'View Testimonial',
                    'search_items' => 'Search Testimonials',
                    'not_found' => 'No testimonials were found.',
                    'not_found_in_trash' => 'No testimonials found in Trash.',
                    'parent_item_colon' => ''
                ),
                'public' => 1,
                'show_ui' => 1,
                'publicly_queryable' => 1,
                'query_var' => 1,
                'rewrite' => 1,
                'capability_type' => 'post',
                'hierarchical' => 1,
                'supports' => array(
                    'title',
                    'editor',
                    'thumbnail',
                    'excerpt',
                    'page-attributes'
                )
            )
        );
    }

    /**
    * add settings page to menu
    *
    */
    function add_settings_menu()
    {
        add_submenu_page( 'edit.php?post_type=testimonial', 'TBTestimonials Settings', 'General Settings', 'manage_options', 'tbtestimonials-settings', array( &$this, 'settings_page' ) );
        $output_syntax_page = add_submenu_page( 'edit.php?post_type=testimonial', 'TBTestimonials Output Syntax Settings', 'Output Settings', 'manage_options', 'tbtestimonials-syntax-settings', array( &$this, 'syntax_page' ) );
        add_action( 'admin_print_scripts-' . $output_syntax_page, create_function( '', 'wp_enqueue_script("CodeMirror");') );
    }

    /**
    * update messages for custom post type
    *
    * @param mixed $m
    */
    function post_update_messages( $m )
    {
        global $post;

        $m['testimonial'] = array(
            0 => '', // Unused. Messages start at index 1.
            1 => sprintf( 'Testimonial updated. <a href="%s">View testimonial</a>', esc_url( get_permalink( $post->ID ) ) ),
            2 => 'Custom field updated.',
            3 => 'Custom field deleted.',
            4 => 'Testimonial updated.',
            /* translators: %s: date and time of the revision */
            5 => isset( $_GET['revision'] ) ? sprintf( 'Testimonial restored to revision from %s', wp_post_revision_title( (int)$_GET['revision'], false ) ) : false,
            6 => sprintf( 'Testimonial published. <a href="%s">View testimonial</a>', esc_url( get_permalink( $post->ID ) ) ),
            7 => 'Testimonial saved.',
            8 => sprintf( 'Testimonial submitted. <a target="_blank" href="%s">Preview testimonial</a>', esc_url( add_query_arg( 'preview', 'true', get_permalink($post->ID) ) ) ),
            9 => sprintf( 'Testimonial scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview testimonial</a>', date_i18n( 'M j, Y @ G:i', strtotime( $post->post_date ) ), esc_url( get_permalink( $post->ID ) ) ),
            10 => sprintf( 'Testimonial draft updated. <a target="_blank" href="%s">Preview testimonial</a>', esc_url( add_query_arg( 'preview', 'true', get_permalink( $post->ID ) ) ) )
        );

        return $m;
    }

    /**
    * add meta boxes to testimonial post types
    *
    */
    function add_meta_boxes()
    {
        if( function_exists( 'add_meta_box' ) )
        {
            add_meta_box( 'tbtestimonial-categories', 'Categories', array( &$this, 'testimonial_categories' ), 'testimonial', 'side', 'low' );
            add_meta_box( 'tbtestimonial-company', 'Company Information', array( &$this, 'company_info' ), 'testimonial', 'side', 'low' );
        }
    }

    /**
    * author meta box
    *
    * @param mixed $post
    */
    function testimonial_categories( $post )
    {
        if( ! $cats = get_option( 'tbtestimonial_categories' ) )
        {
            add_option( 'tbtestimonial_categories', array( 'Uncategorized' ) );
            $cats = get_option( 'tbtestimonial_categories' );
        }
        ?>
        <?php printf( '<input type="hidden" name="tbtestimonial_nonce" id="tbtestimonial_nonce" value="%s" />', wp_create_nonce( plugin_basename( __FILE__ ) ) ); ?>

        <?php
            foreach( $cats as $id => $c )
                printf( '<p id="tbtestimonial-cat-%d"><input type="checkbox" %s name="tbtestimonial_categories[]" id="cat-%d" value="%d" /><label style="margin-left:5px; position:relative; top:-1px;" for="cat-%d">%s</label><a id="tbtestimonials-delete-cat-%d" href="#delete-category" class="tbtestimonials-delete-cat" rel="tbtestimonial-cat-%d">[x]</a></p>', $id, is_array( get_post_meta( $post->ID, 'tbtestimonial_categories', 1 ) ) ? checked( in_array( $id, get_post_meta( $post->ID, 'tbtestimonial_categories', 1 ) ), true, false ) : '', $id, $id, $id, $c, $id, $id );
        ?>

        <span id="tbtestimonials-category-ajax-response"></span>
         <div id="category-adder" class="wp-hidden-children hide-if-no-js">
            <h4><a id="tbtestimonials-category-add-toggle" href="#category-add"><?php _e( '+ Add New Category' ); ?></a></h4>
            <p id="tbtestimonials-link-category-add" class="wp-hidden-child">
                <label class="screen-reader-text" for="tbtestimonials-newcat"><?php _e( '+ Add New Category' ); ?></label>
                <input type="text" name="newcat" id="tbtestimonials-newcat" class="form-required form-input-tip" value="<?php esc_attr_e( 'New category name' ); ?>" aria-required="true" />
                <input type="button" id="tbtestimonials-category-add-submit" class="add:categorychecklist:linkcategorydiv button" value="<?php esc_attr_e( 'Add' ); ?>" />
                <?php wp_nonce_field( 'tbtestimonials_category_ajax_nonce', 'tbtestimonials_category_ajax_nonce', false ); ?>
            </p>
        </div>
        <?php
    }

    function company_info( $post )
    {
        ?><p><label for="testimonial_company">Company Name</label><input style="width:100%" type="text" name="testimonial_company" id="testimonial_company" value="<?php echo get_post_meta( $post->ID, 'tbtestimonial_company', 1 ); ?>" /></p>
        <p><label for="testimonial_company_url">Company URL</label><input style="width:100%" type="text" name="testimonial_company_url" id="testimonial_company_url" value="<?php echo get_post_meta( $post->ID, 'tbtestimonial_company_url', 1 ); ?>" /></p>
        <p><label for="testimonial_company_email">Email Address (<small>For Gravatar</small>)</label><input style="width:100%" type="text" name="testimonial_company_email" id="testimonial_company_email" value="<?php echo get_post_meta( $post->ID, 'tbtestimonial_company_email', 1 ); ?>" /></p><?php
    }

    /**
    * save post data from author meta box
    *
    * @param mixed $id
    */
    function save_postdata( $id )
    {
        if( ! isset( $_POST['tbtestimonial_nonce'] ) || ! wp_verify_nonce( $_POST['tbtestimonial_nonce'], plugin_basename( __FILE__ ) ) )
            return $id;

        if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
            return $id;

        if( 'testimonial' == $_POST['post_type'] )
        {
            if( ! current_user_can( 'edit_page', $id ) )
                return $id;

            if( isset( $_POST['testimonial_company_email'] ) && ! empty( $_POST['testimonial_company_email'] ) )
                update_post_meta( $id, 'tbtestimonial_company_email', strip_tags( $_POST['testimonial_company_email'] ) );

            if( isset( $_POST['testimonial_company_url'] ) && ! empty( $_POST['testimonial_company_url'] ) )
                update_post_meta( $id, 'tbtestimonial_company_url', strip_tags( $_POST['testimonial_company_url'] ) );

            if( isset( $_POST['testimonial_company'] ) && ! empty( $_POST['testimonial_company'] ) )
                update_post_meta( $id, 'tbtestimonial_company', strip_tags( $_POST['testimonial_company'] ) );

            if( isset( $_POST['tbtestimonial_categories'] ) && ! empty( $_POST['tbtestimonial_categories'] ) && is_array( $_POST['tbtestimonial_categories'] ) )
                update_post_meta( $id, 'tbtestimonial_categories', $_POST['tbtestimonial_categories'] );
            else
                delete_post_meta( $id, 'tbtestimonial_categories' );
        }
    }

    /**
    * register settings
    *
    */
    function register_settings()
    {
        register_setting( 'tbtestimonials_settings', 'tbtestimonials_settings' );
        register_setting( 'tbtestimonials_output_settings', 'tbtestimonials_output_settings' );
    }

    /**
    * add testimonial.php template file for single pages
    *
    */
    function maybe_load_testimonial_template()
    {
        global $wp_query;
        $post_type = $wp_query->query_vars['post_type'];
        if( $post_type == 'testimonial' )
        {
            $testimonial = get_posts( array( 'id' => $wp_query->queried_object_id, 'post_type' => 'testimonial' ) );
            if ( ! $testimonial )
            {
                if ( $template = get_404_template() )
                {
                    $wp_query->set_404();
                    status_header( 404 );
                    include( $template );
                    exit;
                }
                else if ( file_exists( TEMPLATEPATH . "/index.php" ) )
                {
                    include( TEMPLATEPATH . "/index.php" );
                    exit;
                }
            }

            if ( $template = $this->get_testimonial_template() )
            {
                include( $template );
                exit;
            }
            else if ( file_exists( TEMPLATEPATH . "/index.php" ) )
            {
                include( TEMPLATEPATH . "/index.php" );
                exit;
            }
        }
    }

    /**
    * get testimonial template
    */
    function get_testimonial_template()
    {
        return get_query_template( 'testimonial' );
    }

    /**
    * callback for ajax add category
    *
    */
    function ajax_add_category()
    {
        if( isset( $_POST['nonce'] ) && wp_verify_nonce( $_POST['nonce'], 'tbtestimonials_category_ajax_nonce' ) && isset( $_POST['catname'] ) && ! empty( $_POST['catname'] ) )
        {
            $response = array();

            $c = strip_tags( $_POST['catname'] );

            if( ! $cats = get_option( 'tbtestimonial_categories' ) )
            {
                add_option( 'tbtestimonial_categories', array( 'Uncategorized' ) );
                $cats = get_option( 'tbtestimonial_categories' );
            }

            # check category doesnt exist already
            foreach( $cats as $cat )
            {
                if( strtolower( $cat ) == strtolower( $c ) )
                {
                    $response['msg'] = sprintf( 'Category "%s" already exists.', ucwords( strtolower( $c ) ) );
                    $response['error'] = 1;
                    print json_encode( (object)$response );
                    die();
                }
            }

            # add category
            $cats[] = $c;

            # find new id of added item
            $tmp = array_flip( $cats );
            $id = array_pop( $tmp );

            update_option( 'tbtestimonial_categories', $cats );

            $response['msg'] = sprintf( 'Successfully added "%s".', $c );
            $response['element'] = sprintf( '<p id="tbtestimonial-cat-%d"><input type="checkbox" name="tbtestimonial_categories[]" id="cat-%d" value="%d" /><label style="margin-left:5px; position:relative; top:-1px;" for="cat-%d">%s</label><a id="tbtestimonials-delete-cat-%d" href="#delete-category" class="tbtestimonials-delete-cat" rel="tbtestimonial-cat-%d">[x]</a></p>', $id, $id, $id, $id, $c, $id, $id );
            $response['delete_element'] = sprintf( '#tbtestimonials-delete-cat-%d', $id );

            print json_encode( (object)$response );
        }
        else
            print json_encode( (object)array( 'msg' => 'You are not authorized to add new categories' ) );

        die();
    }

    /**
    * callback for ajax delete category
    *
    */
    function ajax_delete_category()
    {
        $response = array();
        if( isset( $_POST['catid'] ) && is_numeric( $_POST['catid'] ) && isset( $_POST['nonce'] ) && wp_verify_nonce( $_POST['nonce'], 'tbtestimonials_category_ajax_nonce' ) )
        {
            $id = $_POST['catid'];
            $response['catid'] = $id;
        }
        else
        {
            $response['msg'] = 'You do not have permission to delete categories.';
            print json_encode( (object)$response );
            die();
        }

        if( isset( $id ) )
        {
            if( (int)$id == 0 )
            {
                $response['msg'] = sprintf( 'You can not delete the Uncategorized category' );
                print json_encode( (object)$response );
                die();
            }

            if( $cats = get_option( 'tbtestimonial_categories' ) )
            {
                $tmp = array();
                $cat_name = $cats[ $id ];
                $response['catname'] = $cat_name;

                unset( $cats[ $id ] );

                update_option( 'tbtestimonial_categories', $cats );
            }

            if( (int)$id == 0 ) $response['msg'] = sprintf( 'You can not delete the Uncategorized category' );
        }

        print json_encode( (object)$response );
        die();
    }

    /**
    * ajax callback for widget
    *
    */
    function ajax_get_testimonials()
    {
        $q = new WP_Query(
            array(
                'post_type' => 'testimonial',
                'post_status' => 'publish',
                'showposts' => -1
            )
        );

        $data = array();
        $testimonials = array();
        $tbtcategories = get_option( 'tbtestimonial_categories' );

        if( $widget_options = get_option( 'widget_tbtestimonialswidget' ) )
        {
            foreach( $widget_options as $k => $option )
                if ( is_array( $option ) )
                    if( array_key_exists( 'display_count', $option ) && array_key_exists( 'interval', $option ) )
                        $option_id = $k;

            $data['interval'] = $widget_options[$option_id]['interval'];
            $data['show'] = $widget_options[$option_id]['display_count'];
            $data['loop'] = $widget_options[$option_id]['loop'] == 'on' ? 1:0;

            if( isset( $widget_options[$option_id]['categories'] ) )
                $data['categories'] = $widget_options[$option_id]['categories'];
        }
        else
        {
            $data['interval'] = 4;
            $data['show'] = 4;
            $data['loop'] = 1;
        }


        if( $q->have_posts() )
        {
            while( $q->have_posts() )
            {
                $q->the_post();
                $categories = get_post_meta( get_the_ID(), 'tbtestimonial_categories', 1 );

                if( isset( $data['categories'] ) && is_array( $data['categories'] ) && count( $data['categories'] ) > 0 )
                {
                    if( is_array( $categories ) )
                    {
                        foreach( $data['categories'] as $cat )
                            if( in_array( $cat, $categories ) && ! in_array( $this->prepare_testimonial( 'widget' ), $testimonials ) )
                                $testimonials[] = $this->prepare_testimonial( 'widget' );
                    }
                }
                else
                    $testimonials[] = $this->prepare_testimonial( 'widget' );
            }

            if( count( $testimonials ) < 1 )
            {
                if( count( $data['categories'] ) > 1 )
                {
                    $cat_names = array();
                    foreach( $data['categories'] as $cid )
                        $cat_names[] = $tbtcategories[ $cid ];

                    $testimonials[] = sprintf( 'No testimonials found in categories "%s".', join( '" or "', $cat_names ) );
                }
                else
                    $testimonials[] = sprintf( 'No testimonials found in category "%s".', $tbtcategories[ $data['categories'][0] ] );
            }

            $data['testimonials'] = $testimonials;
        }

        if( function_exists( 'json_encode' ) )
            print json_encode( (object)$data );
        else
            print '{"error":"This plugin requires PHP 5 or greater to work."}';

        die();
    }

    /**
    * load ajax popup window for shortcode.
    */
    function shortcode_window()
    {
        require_once( dirname( __FILE__ ) . '/inc/tinymce3/window.php' );
        die();
    }

    /**
    * handle shortcode
    *
    * @param mixed $atts
    * @return string
    */
    function shortcode( $atts )
    {
        extract(
            shortcode_atts(
                array(
                    'id' => null
                ),
                $atts
            )
        );
        if( ! is_null( $id ) )
        {
            if( strtolower( $id ) == 'all' ) # all testimonials
            {
                $q = new WP_Query( array( 'post_type' => 'testimonial', 'post_status' => 'publish', 'showposts' => '-1', 'order_by' => 'menu_order' ) );

                if( $q->have_posts() )
                {
                    $return = '<div id="tbtestimonial-listing">';
                    while( $q->have_posts() )
                    {
                        $q->the_post();
                        $return .= $this->prepare_testimonial( 'shortcode-all' );
                    }

                    return $return . '</div>';
                }
                else
                    return;
            }
            else # single testimonial
            {
                if( ! is_numeric( $id ) ) return;

                $q = new WP_Query( array( 'p' => $id, 'post_type' => 'testimonial', 'post_status' => 'publish' ) );
                if( $q->have_posts() )
                {
                    while( $q->have_posts() )
                    {
                        $q->the_post();
                        return sprintf( '<div class="tbtestimonial">%s</div>', $this->prepare_testimonial( 'shortcode-single' ) );
                    }
                }
                else
                    return;
            }
        }

        return;
    }

    /**
    * load settings page
    *
    */
    function settings_page()
    {
        tbtestimonials_load_template( 'settings' );
    }

    /**
    * load syntax settings page
    *
    */
    function syntax_page()
    {
        tbtestimonials_load_template( 'syntax-settings' );
    }

    /**
    * return prepared testimonial syntax. must be within loop.
    *
    * @param mixed $type
    * @return mixed
    */
    function prepare_testimonial( $type = 'widget' )
    {
        $tags = array( '%permalink%', '%gravatar%', '%testimonial%', '%author_prefix%', '%author%', '%company_url%', '%company_name%', '%testimonial_excerpt%' );
        $replacements = array(
            get_permalink(),
            get_post_meta( get_the_ID(), 'tbtestimonial_company_email', 1 ) ? get_avatar( get_post_meta( get_the_ID(), 'tbtestimonial_company_email', 1 ), $this->settings['gravatar_size'] ) : get_avatar( 'unknown', $this->settings['gravatar_size'] ),
            apply_filters( 'the_content', get_the_content() ),
            $this->settings['author_prefix'],
            get_the_title(),
            get_post_meta( get_the_ID(), 'tbtestimonial_company_url', 1 ),
            get_post_meta( get_the_ID(), 'tbtestimonial_company', 1 ),
            apply_filters( 'the_excerpt', get_the_excerpt() )
        );

        $tags = apply_filters( 'tbtestimonials_template_tags', $tags );
        $replacements = apply_filters( 'tbtestimonials_template_tag_replacements', $replacements );

        switch( $type )
        {
            case 'widget' : $syntax = apply_filters( 'tbtestimonials_widget_syntax', $this->settings['testimonial_syntax'] ); break;
            case 'shortcode-single' : $syntax = apply_filters( 'tbtestimonials_single_syntax', $this->settings['testimonial_syntax_shortcode'] ); break;
            case 'shortcode-all' : $syntax = apply_filters( 'tbtestimonials_listing_syntax', $this->settings['testimonial_syntax_listing'] ); break;
            default : $syntax = apply_filters( 'tbtestimonials_single_syntax', $this->settings['testimonial_syntax_shortcode'] ); break;
        }

        $ret = str_replace( $tags, $replacements, $syntax );

        # check for user defined conditionals
        if( preg_match_all( '/%if (.+?)%(.+?)??(?:%else%(.+?)??)?%endif%/sim', $ret, $m ) )
        {
            foreach( $m[0] as $k => $pattern )
            {
                switch( trim( $m[1][$k] ) )
                {
                    case 'permalink' : $v = get_permalnk(); break;
                    case 'gravatar' : $v = get_post_meta( get_the_ID(), 'tbtestimonial_company_email', 1 ) ? get_avatar( get_post_meta( get_the_ID(), 'tbtestimonial_company_email', 1 ), $this->settings['gravatar_size'] ) : false; break;
                    case 'testimonial' : $v = apply_filters( 'the_content', get_the_content() ); break;
                    case 'author_prefix' : $v = $this->settings['author_prefix']; break;
                    case 'author' : $v = get_the_title(); break;
                    case 'company_url' : $v = get_post_meta( get_the_ID(), 'tbtestimonial_company_url', 1 ); break;
                    case 'company_name' : $v = get_post_meta( get_the_ID(), 'tbtestimonial_company', 1 ); break;
                    case 'testimonial_excerpt' : $v = apply_filters( 'the_excerpt', get_the_excerpt() ); break;
                    default: $v = false;
                }

                # if var is set, output
                if( $v && ! empty( $v ) )
                    $ret = str_replace( $pattern, trim( $m[2][$k] ), $ret );
                else # output else section if it exists
                {
                    if( isset( $m[3][$k] ) && ! empty( $m[3][$k] ) ) $ret = str_replace( $pattern, $m[3][$k], $ret );
                    else $ret = str_replace( $pattern, '', $ret );
                }
            }

            return $ret;
        }

        return $ret;
    }

    /**
    * get add testimonial form
    *
    * @param mixed $atts
    */
    function testimonial_form( $atts )
    {
        extract( shortcode_atts( array(), $atts ) );
        return $this->settings['add_testimonial_form'];
    }

    /**
    * add codemirror to settings page
    *
    */
    function add_codemirror()
    {
        $js_url = plugins_url( 'inc/js/', __FILE__ );
        $css_url = plugins_url( 'inc/css/', __FILE__ );
        $parser = '["parsexml.js"]';
        $styles = '["'.$css_url.'xmlcolors.css"]';
        ?>
            <script type="text/javascript">
                var editor1 = CodeMirror.fromTextArea('testimonial-syntax', {
                  height: "205px",
                  parserfile: <?php echo $parser; ?>,
                  stylesheet: <?php echo $styles; ?>,
                  path:"<?php echo $js_url; ?>",
                  continuousScanning: 500,
               });

               var editor2 = CodeMirror.fromTextArea('testimonial-syntax-shortcode', {
                  height: "205px",
                  parserfile: <?php echo $parser; ?>,
                  stylesheet: <?php echo $styles; ?>,
                  path:"<?php echo $js_url; ?>",
                  continuousScanning: 500,
               });

               var editor3 = CodeMirror.fromTextArea('testimonial-syntax-listing', {
                  height: "205px",
                  parserfile: <?php echo $parser; ?>,
                  stylesheet: <?php echo $styles; ?>,
                  path:"<?php echo $js_url; ?>",
                  continuousScanning: 500,
               });

            </script>
        <?php
    }

    /**
    * check for a bug in earlier versions that had a double quote in the output
    *
    */
    function quote_bug_exists()
    {
        $bug = false;
        if( preg_match( '/testimonial-data"">/si', $this->settings['testimonial_syntax'] ) ) $bug = true;
        if( preg_match( '/testimonial-data"">/si', $this->settings['testimonial_syntax_shortcode'] ) ) $bug = true;
        if( preg_match( '/testimonial-data"">/si', $this->settings['testimonial_syntax_listing'] ) ) $bug = true;
        return $bug;
    }

    /**
    * fix double quote bug. we do it this way as to not overwrite any changes made by existing users
    *
    */
    function fix_quote_bug()
    {
        $new = array();
        $new['testimonial_syntax'] = preg_replace( 'testimonial-data""', 'testimonial-data"', $this->settings['testimonial_syntax'] );
        $new['testimonial_syntax_shortcode'] = preg_replace( 'testimonial-data""', 'testimonial-data"', $this->settings['testimonial_syntax_shortcode'] );
        $new['testimonial_syntax_listing'] = preg_replace( 'testimonial-data""', 'testimonial-data"', $this->settings['testimonial_syntax_listing'] );
        update_option( 'tbtestimonials_output_settings', $new );
        $this->settings = tbtestimonials_settings();
    }

}

/**
*   widget class
*/
class TBTestimonialsWidget extends WP_Widget
{
    static $title = null,
           $display_count = null,
           $interval = null,
           $before_title,
           $after_title;

    /**
    * constructor
    *
    * @param mixed $a
    * @param mixed $i
    */
    function TBTestimonialsWidget()
    {
        parent::WP_Widget( false, $name = 'TB Testimonials Widget' );
    }

    /**
    * widget output
    *
    * @param mixed $a
    * @param mixed $i
    */
    function widget( $a, $i )
    {
        extract( $a );
        self::$before_title = $before_title;
        self::$after_title = $after_title;
        self::$title = empty( $i['title']) ? '&nbsp;' : apply_filters( 'tbtestimonials_widget_title', $i['title'] );
        self::$display_count = empty( $i['display_count']) ? '' : apply_filters( 'tbtestimonials_widget_display_count', $i['display_count'] );
        self::$interval = empty( $i['interval']) ? '&nbsp;' : apply_filters( 'tbtestimonials_widget_interval', $i['interval'] );
        tbtestimonials_load_template( 'widget' );
    }

    /**
    * update routine
    *
    * @param mixed $n - new instance
    * @param mixed $o - old instance
    */
    function update( $n, $o )
    {

        $i = $o;
        $i['title'] = strip_tags( $n['title'] );
        $i['display_count'] = filter_var( $n['display_count'], FILTER_VALIDATE_INT );
        $i['interval'] = filter_var( $n['interval'], FILTER_VALIDATE_INT );
        $i['loop'] = $n['loop'];

        if( ! $cats = get_option( 'tbtestimonial_categories' ) )
        {
            add_option( 'tbtestimonial_categories', array( 'Uncategorized' ) );
            $cats = get_option( 'tbtestimonial_categories' );
        }

        unset( $i['categories'] );
        foreach( $cats as $id => $c )
            if( array_key_exists( $c, $n ) )
                $i['categories'][] = $id;


        return $i;
    }

    /**
    * update form
    *
    * @param mixed $i
    */
    function form( $i )
    {
        $i = wp_parse_args( (array)$i, array( 'title' => 'Testimonials', 'display_count' => '4', 'interval' => '4' ) );
        $title = $i['title'];
        $display_count = filter_var( $i['display_count'], FILTER_VALIDATE_INT );
        $interval = filter_var( $i['interval'], FILTER_VALIDATE_INT );
        $loop = isset( $i['loop'] ) ? $i['loop'] : false; ?>
        <p><label for="<?php echo $this->get_field_id('title'); ?>">Title: <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" /></label></p>
        <p><label for="<?php echo $this->get_field_id('display_count'); ?>">Display Count <small>( -1 shows all testimonials )</small>: <input class="widefat" id="<?php echo $this->get_field_id('display_count'); ?>" name="<?php echo $this->get_field_name('display_count'); ?>" type="text" value="<?php echo esc_attr( $display_count ); ?>" /></label></p>
        <p><label for="<?php echo $this->get_field_id('interval'); ?>">Timer Interval: <input class="widefat" id="<?php echo $this->get_field_id('interval'); ?>" name="<?php echo $this->get_field_name('interval'); ?>" type="text" value="<?php echo esc_attr($interval); ?>" /></label></p>
        <p><label for="<?php echo $this->get_field_id('loop'); ?>"><input class="widefat" id="<?php echo $this->get_field_id('loop'); ?>" name="<?php echo $this->get_field_name('loop'); ?>" type="checkbox" <?php checked( $loop, 'on' ); ?>/> Loop through all testimonials</label></p>
        <h4>Categories to display<br /><small>(Select none to show all)</small></h4><?php

        if( ! $cats = get_option( 'tbtestimonial_categories' ) )
        {
            add_option( 'tbtestimonial_categories', array( 'Uncategorized' ) );
            $cats = get_option( 'tbtestimonial_categories' );
        }

        if( ! isset( $i['categories'] ) || ! is_array( $i['categories'] ) ) $i['categories'] = array();

        foreach( $cats as $id => $cat )
            printf( '<p><input type="checkbox" %s name="%s" id="%s" value="%d" /><label style="margin-left:5px; position:relative; top:-1px;" for="%s">%s</label></p>', checked( in_array( $id, $i['categories'] ), true, 0 ), $this->get_field_name( $cat ), $this->get_field_id('tbcat' . $id ), $id, $this->get_field_id('tbcat' . $id ), $cat );
    }
}

# init widget
add_action( 'widgets_init', create_function( '', 'return register_widget( "TBTestimonialsWidget" );' ) );

# init plugin
add_action( 'init', create_function( '', 'return $tbtestimonials = new TBTestimonials();' ) );

# tinymce button
if( is_admin() )
{
    require_once( dirname( __FILE__ ) . '/inc/tinymce3/tinymce.php' );
    $tbtestimonials_tinymce_button = new TBTestimonialsTinyMCEButton();
}


/**
* function to output testimonial programatically into theme
*
* @param mixed $id
*/
function tbtestimonial( $id = false, $echo = true )
{
    if( $id === false || ! is_numeric( $id ) ) return false;

    $tbtestimonials = new TBTestimonials();
    $q = new WP_Query( array( 'p' => $id, 'post_type' => 'testimonial', 'post_status' => 'publish' ) );

    if( $q->have_posts() )
    {
        while( $q->have_posts() )
        {
            $q->the_post();
            $output = sprintf( '<div class="tbtestimonial">%s</div>', $tbtestimonials->prepare_testimonial( 'shortcode-single' ) );
        }

        $output = apply_filters( 'tbtestimonials_single_syntax', $output );
        if( ! $echo ) return $output; else print $output;
    }
    else
        return false;
}


/**
* load a template file from cwd. seperate php and markup this way.
*
* @param string $file
*/
function tbtestimonials_load_template( $file )
{
    $file = sprintf( '%s/inc/tpl/%s.php', dirname( __FILE__ ), $file );
    if( file_exists( $file ) && is_readable( $file ) )
        include( $file );
    else
    {
        if( ! is_readable( $file ) && file_exists( $file ) )
            wp_die( sprintf( 'Unable to load template file %s. File is not readable. Check permissions and try again.', $file ) );
        elseif( ! file_exists( $file ) )
            wp_die( sprintf( 'Unable to load template file %s. File does no exist.', $file ) );
    }
}

/**
* get settings or add them if needed.
*
*/
function tbtestimonials_settings()
{
    if( ! $general_settings = get_option( 'tbtestimonials_settings' ) )
    {
        add_option( 'tbtestimonials_settings', array(
            'author_prefix' => '&mdash;',
            'use_stylesheet' => 1,
            'show_loading_graphic' => 1,
            'loading_graphic_url' => plugins_url( 'inc/i/loaders/loader_default.gif', __FILE__ ),
            'loading_text' => 'Loading',
            'loading_text_position' => 'after',
            'gravatar_size' => 50
        ) );

        $general_settings = get_option( 'tbtestimonials_settings' );
    }

    if( ! $output_settings = get_option( 'tbtestimonials_output_settings' ) )
    {
        add_option( 'tbtestimonials_output_settings', array( 'testimonial_syntax' => '<li class="testimonial">
    <div class="testimonial-gravatar">%gravatar%</div>
    <div class="testimonial-data">
        <p class="testimonial-content">%testimonial_excerpt%</p>
        <p class="testimonial-author">%author_prefix%%author%</p>
        %if company_url%
            <p class="testimonial-company"><a href="%company_url%">%company_name%</a></p>
        %endif%
    </div>
    <div class="clear"></div>
</li>',
            'testimonial_syntax_shortcode' => '<div class="in-content-testimonial">
    <div class="testimonial-gravatar">%gravatar%</div>
    <div class="testimonial-data">
        <p class="testimonial-content">%testimonial%</p>
        <p class="testimonial-author">%author_prefix%%author%</p>
        %if company_url%
            <p class="testimonial-company"><a href="%company_url%">%company_name%</a></p>
        %endif%
    </div>
    <div class="clear"></div>
</div>
',
            'testimonial_syntax_listing' => '<div class="in-listing-testimonial">
    <div class="testimonial-gravatar">%gravatar%</div>
    <div class="testimonial-data">
        <p class="testimonial-content">%testimonial%</p>
        <p class="testimonial-author">%author_prefix%%author%</p>
        %if company_url%
            <p class="testimonial-company"><a href="%company_url%">%company_name%</a></p>
        %endif%
    </div>
    <div class="clear"></div>
</div>
',
            'add_testimonial_form' => '<form action="" method="post" class="add-testimonial" id="tbtestimonials-insert-testimonial-form">
    <p><label for="name">Name:</label><input id="name" type="text" size"40" name="tbtestimonial_name" /></p>
    <p><label for="company_name">Company Name:</label><input id="company_name" type="text" size"40" name="tbtestimonial_company_name" /></p>
    <p><label for="company_url">Website URL:</label><input id="company_url" type="text" size"40" name="tbtestimonial_company_url" /></p>
    <p><label for="email_address">Email Address</label><input id="email_address" type="text" size"40" name="tbtestimonial_email_address" /></p>
    <p><label for="testimonial">Testimonial</label><textarea name="testimonial" id="testimonial" cols="100" rows="10"></textarea></p>
    <p><input type="submit" value="Submit" /></p>
</form>'
 ) );

        $output_settings = get_option( 'tbtestimonials_output_settings' );
    }

    return array_merge( $general_settings, $output_settings );
}
